import com.mwc.util.CListint;

public class IntCListTest {
   public static void main(String args[]) {
      CListint intlist = new CListint();
      intlist.pushBack(1);
      intlist.pushBack(2);
      intlist.pushBack(3);
      intlist.pushFront(-1);
      intlist.pushFront(-2);
      intlist.pushFront(-3);
      for(CListint.Iterator ptr = intlist.begin(); ptr.notEnd(); ptr.next())
         System.out.println(ptr.data());
   }
}